package main;

import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;

import method.dictionary.SearchSeedTerms;
import method.signature.SignatureSimilarity;

/**
 * @param args
 * @throws IOException 
 * 
 * Biomedical Named Entity Recognizer
 */
public class Ner {
	/**
	 * 
	 * @param args
	 * 	 arg 0 = "seed": seed term collection mode
	 * 		arg 1: configuration file for seed term collection
	 * 	 arg 0 = "recognize": ner mode
	 * 		arg 1: file name of the backup corpus for signature generation
	 * 		arg 2: file name of the test corpus
	 * 		arg 3: how many context words included before and after respectively
	 * 		arg 4: similarity threshold for entity classification
	 * 		arg 5: inner word weight
	 * 		arg 6: 1: regenerate signatures, use when change parameters or first use; 0: use existing signaturess
	 * @throws IOException
	 * @throws SAXException
	 * @throws ParserConfigurationException
	 */
	public static void main(String[] args) throws IOException, SAXException, ParserConfigurationException {
		if(args[0].equals("seed")) {
			if(args[1].equals("-c")) {
				String config = args[2];
				SearchSeedTerms finder = new SearchSeedTerms(config);
				finder.search();
			}
			else {
				System.err.println("no configuration file, see http://people.dbmi.columbia.edu/~szhang/ner.html for help");
			}
		}
		else if(args[0].equals("recognize")) {
			String backupCorpus = "";
			String testCorpus = "";
			int context = 2;
			double threshold = 0.002;
			double inner = 20;
			int reg = 1;
			
			for(int i = 1; i < args.length; ++ i) {
				if(args[i].equals("-b")) {
					backupCorpus = args[i+1];
					i++;
					continue;
				}
				if(args[i].equals("-f")) {
					testCorpus = args[i+1];
					i ++;
					continue;
				}
				if(args[i].equals("-c")) {
					context = Integer.parseInt(args[i+1]);
					i ++;
					continue;
				}
				if(args[i].equals("-t")) {
					threshold = Double.parseDouble(args[i+1]);
					++ i;
					continue;
				}
				if(args[i].equals("-i")) {
					inner = Double.parseDouble(args[i+1]);
					++ i;
					continue;
				}
				if(args[i].equals("-r")) {
					reg = Integer.parseInt(args[6]);
					++ i;
					continue;
				}
			}
			
			boolean regenerate = reg==1?true:false;
			
			System.out.println("building signatures...");
			SignatureSimilarity model = new SignatureSimilarity(backupCorpus, context, threshold, inner, regenerate);

			model.recognize(testCorpus);
		}
		else {
			System.err.println("incorrect configuration");
		}
		
	}

}
