package method.dictionary;

import java.io.File;
import java.io.IOException;
import java.util.HashSet;
import java.util.Hashtable;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class DomainConfig {
	
	private HashSet<String> classes;
	private Hashtable<String, HashSet<String>> class_type_mapping;
	private Hashtable<String, HashSet<String>> class_concept_mapping;
	
	public DomainConfig() {
		classes = new HashSet<String>();
		class_type_mapping = new Hashtable<String, HashSet<String>>();
		class_concept_mapping = new Hashtable<String, HashSet<String>>();
	}
	
	/**
	 * @return set of all entity classes
	 */
	public HashSet<String> getClasses() {
		return classes;
	}
	
	/**
	 * @return mapping from entity class to all corresponding semantic types in domain representation
	 */
	public Hashtable<String, HashSet<String>> getClassTypeMapping(){
		return class_type_mapping;
	}
	
	/**
	 * @return maping from entity class to all corresponding UMLS concepts in domain representation
	 */
	public Hashtable<String, HashSet<String>> getClassConceptMapping(){
		return class_concept_mapping;
	}
	
	/**
	 * @param filename XML configuration file recording mappings from entity classes to semantic types/groups or concepts
	 * @throws ParserConfigurationException
	 * @throws SAXException
	 * @throws IOException
	 */
	public void readConfigFile(String filename) throws ParserConfigurationException, SAXException, IOException {
		File fXmlFile = new File(filename);
		DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
		DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
		Document doc = dBuilder.parse(fXmlFile);
		doc.getDocumentElement().normalize();

		NodeList nList = doc.getElementsByTagName("class");
		for (int temp = 0; temp < nList.getLength(); temp++) {
			Node nNode = nList.item(temp);
			String entity = nNode.getAttributes().getNamedItem("name").getNodeValue();
			
			classes.add(entity);
			
			class_type_mapping.put(entity, new HashSet<String>());
			class_concept_mapping.put(entity, new HashSet<String>());
			
			NodeList mapList = nNode.getChildNodes();
			for(int i = 0; i < mapList.getLength(); ++ i) {
				Node mapNode = mapList.item(i);
				
				if(mapNode.getNodeType() == Node.ELEMENT_NODE) {
					String type = mapNode.getAttributes().getNamedItem("type").getNodeValue();
					if(type.equals("semantic type")) {
						class_type_mapping.get(entity).add(mapNode.getTextContent());
					}
					else if(type.equals("concept")) {
						class_concept_mapping.get(entity).add(mapNode.getTextContent());
					}
				}		
			}
		}
	}
	
}
