package method.signature;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;

import util.RegularExpression;

public class SignatureGenerator {

	HashSet<String> stopwords;
	Hashtable<String, Double> IDF = new Hashtable<String, Double>();

	public SignatureGenerator(Hashtable<String, Double> IDF) throws IOException {
		stopwords = new HashSet<String>();
		BufferedReader reader = new BufferedReader(new FileReader("stopwords.txt"));
		while(reader.ready()) {
			stopwords.add(reader.readLine());
		}
		this.IDF = IDF;
	}
	
	public Hashtable<String, Double> generateSignatureForTerm(int begin, int end, String[] sentence, int context, double inner) {
		return generateFrequencySignatureForTerm(begin, end, sentence, context, inner);
	}
	
	public Hashtable<String, Double> combineSignature(HashSet<Hashtable<String, Double>> set) {
		Hashtable<String, Double> table = new Hashtable<String, Double>();
		
		Iterator<Hashtable<String, Double>> ite = set.iterator();
		while(ite.hasNext()){
			Hashtable<String, Double> tmp = ite.next();
			Enumeration<String> enu = tmp.keys();
			while(enu.hasMoreElements()) {
				String key = enu.nextElement();
				if(table.containsKey(key)) {
					table.put(key, table.get(key) + tmp.get(key));
				}
				else {
					table.put(key, tmp.get(key));
				}
			}
		}
		return table;
	}
	
	private Hashtable<String, Double> generateFrequencySignatureForTerm(int begin, int end, String[] sentence, int context, double inner) {
		Hashtable<String, Double> dic = new Hashtable<String, Double>();
		for(int i = begin - context; i < end + context + 1; ++ i) {
			
			if(i < 0 || i >= sentence.length)
				continue;
			String word = sentence[i];
			if(stopwords.contains(sentence[i]))
				continue;

			if(RegularExpression.isNumber(sentence[i]))
				word = "DIGIT";
			
			Double weight = IDF.get(word);
			if(weight == null)
				weight = 0.;
			if(i >= begin && i <= end) {
				word = "IN_" + word;
				weight = weight*inner;
			}
					
			if(dic.containsKey(word)) {
				double score =  dic.get(word)+weight;
				dic.put(word, score);
			}
			else {
				dic.put(word, weight);
			}
		}
		return dic;
	}
	
	public Hashtable<String, Double> generateFrequencySignatureForSet(HashSet<String> termSet, String file, int context, double inner, boolean includeUnigram) throws IOException {
		BufferedReader reader = new BufferedReader(new FileReader(file));
		Hashtable<String, Double> dic = new Hashtable<String, Double>();
		
		//number of occurrence. Remove those ones occurs only once in the dataset.
		Hashtable<String, Integer> occur = new Hashtable<String, Integer>();
		
		double counter = 0;
		
		if(includeUnigram) {
		//Put word in the terms into the signature no matter if they appear in the dataset
			for(String s : termSet) {
				String[] tokens = s.split(" ");
				for(String token : tokens) {
					String word  = "IN_" + token;
					if(stopwords.contains(word))
						continue;
					
					Double weight = IDF.get(word);
					if(weight == null)
						weight = 1.;
					if(dic.containsKey(word)) {
						double value = dic.get(word)+weight;
						dic.put(word, value);
					}
					else {
						dic.put(word, weight);
					}
				}
			}
		}
		
		while(reader.ready()) {
			String line = reader.readLine();
			line = line.replace('\t', ' ');
			line = line.toLowerCase();
			String[] tokens = line.split(" ");
			
			for(int i = 0; i < tokens.length; ++ i) {
				for(int j = i; j < tokens.length && j < i + 6; ++ j) {
					String candidate = "";
					for(int k = i; k < j; ++ k) {
						candidate += tokens[k] + " ";
					}
					candidate += tokens[j];
					if(termSet.contains(candidate) && !stopwords.contains(candidate)) {
						counter ++;
						for(int m = i -context; m <= j + context; ++ m) {
							if(m < 0 || m >= tokens.length)
								continue;

							String word = tokens[m];
							if(stopwords.contains(tokens[m]))
								continue;
							if(RegularExpression.isNumber(tokens[m]))
								word = "DIGIT";
							
							double weight = IDF.get(word);
							if(m >= i && m <= j) {
								word = "IN_" + word;
								weight = weight*inner;
							}
							
							if(!occur.containsKey(word))
								occur.put(word, 1);
							else
								occur.put(word, occur.get(word) + 1);
							
							if(dic.containsKey(word)) {
								double value = dic.get(word)+weight;
								dic.put(word, value);
							}
							else {
								dic.put(word, weight);
							}
						}
					}
				}
			}
		}
		
		Enumeration<String> en = dic.keys();
		while(en.hasMoreElements()) {
			String word = en.nextElement();
			dic.put(word, dic.get(word)/counter);
		}
		
		return dic;
	}

}
